<?php

// We don't want to allow direct access to this
defined( 'ABSPATH' ) OR die( 'No direct script access allowed' );

// we also only want to allow CLI access
defined( 'WP_CLI' ) OR die( 'Only CLI access allowed' );

class KCP_CSPGEN_CLI_Parser {

    public function parse( string $_body ) : array {

        // if we have an empty body string, there's nothing to parse, so just skip it
        if( empty( $_body ) ) {

            // return an empty array here, since we are declaring the return type of this method
            return array( );
        }

        // setup the returnable array
        $_tmp = array( );

        // setup an array of html tags we want to look for
        $_tags = array( 'form', 'link', 'script', 'frame', 'iframe', 'img', 'object', 'video', 'audio', 'embed', 'source' );

        // lets hold a few more
        $_tmp[ 'css' ] = array( );
        $_tmp[ 'script' ] = array( );
        $_tmp[ 'connect' ] = array( );
        $_tmp[ 'frame' ] = array( );
        $_tmp[ 'media' ] = array( );
        $_tmp[ 'font' ] = array( );
        $_tmp[ 'img' ] = array( );
        $_tmp[ 'default' ] = array( );
        $_tmp[ 'form' ] = array( );
        
        // let's utilize the built-in PHP DomDocument to parse
        $_doc = new DOMDocument( );

        // silence
        libxml_use_internal_errors( true );

        // load our html into the parser
        $_doc -> loadHTML( $_body );

        // clear the libxml errors
        libxml_clear_errors( );

        // we need to loop the tags array and process it
        foreach( $_tags as $_tag) {

            // get the element
            $_elements = $_doc -> getElementsByTagName( $_tag );

            // make sure it actually exists before proceeding
            if( $_elements ) {

                // loop over the elements
                foreach( $_elements as $_element ) {

                    // switch for the proper URL's to gather based on the tag we're looking at
                    switch( $_tag ) {
                        case 'img': // images
                            array_push( $_tmp['img'], $_element -> getAttribute( 'src' ) );
                            break;
                        case 'form': // forms
                            array_push( $_tmp['form'], $_element -> getAttribute( 'action' ) );
                            break;
                        case 'script': // scripts
                            array_push( $_tmp['script'], $_element -> getAttribute( 'src' ) );
                            break;
                        case 'link':

                            // check if the link is a stylesheet or something else
                            if( $_element -> getAttribute( 'rel' ) == 'stylesheet' ) {

                                // it is, so push or to the styles array
                                array_push( $_tmp['css'], $_element -> getAttribute( 'href' ) );

                                // lets parse through the CSS and see if we have any fonts
                                $this -> parse_css_for_fonts( $_element -> getAttribute( 'href' ), $_tmp );

                            } elseif( ( $_element -> getAttribute( 'rel' ) == 'icon' ) || ( $_element -> getAttribute( 'rel' ) == 'apple-touch-icon-precomposed' ) || ( $_element -> getAttribute( 'rel' ) == 'apple-touch-icon' ) ) {

                                // it's a favicon
                                array_push( $_tmp['img'], $_element -> getAttribute( 'href' ) );
                            } elseif( ( $_element -> getAttribute( 'rel' ) != 'shortlink' ) && ( $_element -> getAttribute( 'rel' ) != 'sitemap' ) ) {

                                // it's something to connect
                                array_push( $_tmp['connect'], $_element -> getAttribute( 'href' ) );
                            }
                            break;
                        case 'frame':
                        case 'iframe': //frames
                            array_push( $_tmp['frame'], $_element -> getAttribute( 'src' ) );
                            break;
                        case 'video':
                        case 'audio':
                        case 'source':
                            array_push( $_tmp['media'], $_element -> getAttribute( 'src' ) );
                            break;
                        case 'embed':
                            // we'll need to check what type of file is being included in the source
                            $_src = $_element -> getAttribute( 'src' );

                            // parse the embed
                            $this -> parse_objectembed( $_src, $_tmp );
                            break;
                        case 'object':
                            // we'll need to check what type of file is being included in the source
                            $_src = $_element -> getAttribute( 'data' );

                            // parse the emdbed
                            $this -> parse_objectembed( $_src, $_tmp );
                            break;
                    }

                }

            }

        }

        // release the dom parser
        unset( $_doc );

        // return it
        return $_tmp;
    }

    // parse the CSS for a font to store
    protected function parse_css_for_fonts( string $_src, array &$_arr ) : void {

        // make sure the URL has the protocol
        if( ! parse_url( $_src, PHP_URL_SCHEME ) ) {
            $_src = 'https:' . $_src;
        }

        // request the file
        $_resp = wp_safe_remote_get( $_src, array( 'timeout' => 45, ) );

        // make sure we have a resposne
        if( is_array( $_resp ) && ! is_wp_error( $_resp ) ) {

            // populate the body so we can parse the CSS
            $_body = $_resp['body'];

            // hold the matches
            $_matches = array( );

            // try to get all our font-faces
            preg_match_all( '/^(.*):\s*(.*)\s*;$/m', $_body, $_matches, PREG_SET_ORDER );

            // hold our parsed out info
            $_parsed = array( );

            // setup a count
            $_ct = 0;

            // loop over the matches
            foreach( $_matches as $_k => $_match ) {

                // if we've got the first item, setup something to hold a temporary array
                if($_ct === 0) {
                    // here
                    $tmp = array();
                }

                $tmp[ trim( $_match[ 1 ] ) ] = trim( $_match[ 2 ] );

                if ( $_ct === 3 ) {

                    $_parsed[] = $tmp;
                    $_ct = 0;

                } else {

                    $_ct ++;

                }
            }

            // parse through the parsed out fonts
            if( count( $_parsed ) > 0 ) {

                // loop over them
                foreach( $_parsed as $_p ) {

                    // force numeric indeces
                    $_p = array_values( $_p );

                    // the temp source
                    $_tmp_source = ( $_p[3] ) ?? null;

                    // parse the hostname out
                    $_host = parse_url( $_tmp_source, PHP_URL_HOST );

                    // append the host to the array
                    array_push( $_arr['font'], $_host );

                }
            }

        }

    }

    // parse the object for the type of object we need to store
    protected function parse_objectembed( string $_src, array &$_arr ) : void {

        // set a default
        $_content_type = 'plain/text';

        // set a couple arrays to hold mimetypes
        $_img = array( 'image/png', 'image/jpg', 'image/jpeg', 'image/bmp', 'image/gif', 'image/tiff', 'image/svg+xml', 'image/webp', 'image/apng', 'image/x-icon' );
        $_media = array( 'audio/wave', 'audio/wav', 'audio/x-wav', 'audio/x-pn-wav', 'audio/webm', 'audio/ogg', 'application/ogg', 'video/ogg', 'video/webm', 'video/x-flv', 'video/mp4', 'application/x-mpegURL', 'video/MP2T', 'video/3gpp', 'video/quicktime', 'video/mkv', 'video/x-msvideo', 'video/x-ms-wmv' );
        $_css = array( 'text/css' );
        $_script = array( 'application/javascript', 'application/ecmascript', 'application/x-ecmascript', 'application/x-javascript', 'text/javascript', 'text/ecmascript', 'text/javascript1.0', 'text/javascript1.1', 'text/javascript1.2', 'text/javascript1.3', 'text/javascript1.4', 'text/javascript1.5', 'text/jscript', 'text/livescript', 'text/x-ecmascript', 'text/x-javascript' );

        // make a request for just the headers on this item.  we only need the content-type returned
        $_r = wp_remote_head( $_src );

        // as long as its valid response
        if ( is_array( $_r ) && ! is_wp_error( $_r ) ) {

            // get the content type
            $_content_type = wp_remote_retrieve_header( $_r, 'content-type' );

            // check if we're in the image array
            if( in_array( $_content_type, $_img ) ) {

                // yep, we have an image
                array_push( $_arr['img'], $_src );
            } elseif( in_array( $_content_type, $_media ) ) { // check if we're in the media array

                // yep, we have media
                array_push( $_arr['media'], $_src );
            } elseif( in_array( $_content_type, $_css ) ) { // check if we're in the css array

                // yep, we have css
                array_push( $_arr['css'], $_src );
            } elseif( in_array( $_content_type, $_script ) ) { // check if we're in the script array

                // yep, we have script
                array_push( $_arr['script'], $_src );
            } else {

                // throw it in the default
                array_push( $_arr['default'], $_src );
            }

        }

    }

}