<?php

// We don't want to allow direct access to this
defined( 'ABSPATH' ) OR die( 'No direct script access allowed' );

// we also only want to allow CLI access
defined( 'WP_CLI' ) OR die( 'Only CLI access allowed' );

class KCP_CSPGEN_CLI_Requests {

    // setup a couple internal properties
    private string $ba_user;
    private string $ba_pass;

    // fire us up
    public function __construct( ) {

        // populate the internal properties
        $this -> ba_user = ( get_our_option( 'auth_un' ) ) ?? null;
        $this -> ba_pass = ( get_our_option( 'auth_pw' ) ) ?? null;

    }

    // do what we need with the pages we have
    public function cli_requests( array $_pages ) : array {

        // hold our return
        $_ret = array( );

        // loop over the pages and pull them
        foreach( $_pages as $_url ) {

            // perform the request
            $_page_body = $this -> cli_curl( $_url );

            // now that we have the body we need to parse it, so do just that
            $_parser = new KCP_CSPGEN_CLI_Parser( );

            // setup the return property, make sure they are unique
            $_ret[] = $_parser -> parse( $_page_body );

        }

        // return the array
        return $_ret;
    }

    // use the wordpress HTTP API to make our requests
    protected function cli_curl( string $_url ) : ?string {

        // hold our response string
        $_resp = '';

        // setup our arguments
        $_args = array(
            'timeout' => 20,
            'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0',
        );

        // if we need to authenticate, merge that header in
        if( $this -> ba_user && $this -> ba_pass ) {

            // merge the auth header
            $_args = array_merge( $_args, array(
                'headers' => array(
                    'Authorization' => 'Basic ' . base64_encode( $this -> ba_user . ':' . $this -> ba_pass )
                )
            ) );
        }

        // make the request
        $_req = wp_remote_get( $_url, $_args );

        // make sure we have a valid response code before proceeding
        if( wp_remote_retrieve_response_code( $_req ) == 200 ) {

            // we do, so toss in our response
            $_resp = wp_remote_retrieve_body( $_req );
        }

        // return our response
        return $_resp;

    }

}