<?php

// We don't want to allow direct access to this
defined( 'ABSPATH' ) OR die( 'No direct script access allowed' );

// Plugin Activation
register_activation_hook( WPSH_PATH . '/' . WPSH_FILENAME, function( $_network )  {

    // check if we tried to network activate this plugin
    if( is_multisite( ) && $_network ) {

        // we did, so... throw an error message and exit
        wp_die( __( '<h1>Cannot Network Activate</h1><p>Due to the nature of this plugin, it cannot be network activated.</p><p>Please go back, and activate inside your subsites.</p>' ), 
            __( 'Cannot Network Activate' ),
            array(
                'back_link' => true,
            ) );
    }

} );

// Plugin De-Activation
register_deactivation_hook( WPSH_PATH . '/' . WPSH_FILENAME, function( ) {

} );

// make sure this plugin is actually active before we do anything
if( in_array( WPSH_DIRNAME . '/' . WPSH_FILENAME, apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {

    // setup a common function to get one of our options
    function get_our_option( string $_opt ) {

        // return the option, or null if it does not exist
        return get_option( '_kp_cspgen_' . $_opt ) ?? null;

    }    

    // setup our autoload
    spl_autoload_register( function( $_cls ) {

        // reformat the class name to match the file name for inclusion
        $_class = strtolower( str_ireplace( '_', '-', $_cls ) );

        // check if we're in CLI or not
        if( defined( 'WP_CLI' ) ) {

            // we are, so let's pull the CLI classes
            $_path = WPSH_PATH . '/cli/work/' . $_class . '.php';
        } else {

            // we're not, so let's pull the non-cli classes
            $_path = WPSH_PATH . '/work/inc/' . $_class . '.php';
        }

        // if the file exists
        if( file_exists( $_path ) ) {

            // include it once
            include_once( $_path );
        }

    } );

    // let's see if we're in CLI or not
    if( ! defined( 'WP_CLI' ) ) {

        // load in our Carbon Fields library
        add_action( 'after_setup_theme', function( ) {

            // only load if it's already non-existent
            if( ! class_exists( 'Carbon_Fields\Carbon_Fields' ) ) {

                // require the autoloader
                require_once( WPSH_PATH . '/vendor/autoload.php' );

                // boot Carbon Fields
                \Carbon_Fields\Carbon_Fields::boot( );

            }

        } );

        // hook into cf to fire up the admin pages
        add_action( 'carbon_fields_register_fields', function( ) { 

            // create the settings as well as the menu pages
            $_settings = new KCP_CSPGEN_Settings( );
            $_settings -> kp_cspgen_settings( );

            // clean it up
            unset( $_settings );

        }, PHP_INT_MAX );

        // hack in some styling
        add_action( 'admin_enqueue_scripts', function( ) {

            // register the stylesheet
            wp_register_style( 'kpsh_css', plugins_url( '/style.css', WPSH_PATH . '/' . WPSH_FILENAME ), null, null );

            // enqueue it
            wp_enqueue_style( 'kpsh_css' );

            
        }, PHP_INT_MAX );

        // bring in our header functionality and apply them
        $_headers = new KCP_CSPGEN_Headers( );
        $_headers -> kp_process_headers( );

        // dump it
        unset( $_headers );

    } else {

        // fire off our CLI implementation
        new KCP_CSPGEN_CLI( );

    }

}