<?php

// We don't want to allow direct access to this
defined( 'ABSPATH' ) OR die( 'No direct script access allowed' );

// check user capabilities
if( ! current_user_can( 'manage_options' ) ) {
    return;
}
?>

<style type="text/css">
    .wrap ul {
        margin-left: 35px;
    }
</style>
    <p>This plugin generates the proper security HTTP response headers and generates a Content Security Policy if configured to do so.</p>
    <p>In order to grab everything possible, there is a CLI only component that will parse the entire website for external resources and store the items found in a custom post.</p>

    <h3 id="install">Install</h3>
    <ul>
        <li>Download the plugin, unzip it, and upload to your sites `/wp-content/plugins/` directory
            <ul>
                <li>You can also upload it directly to your Plugins admin</li>
            </ul>
        </li>
        <li>Activate the plugin through the 'Plugins' menu in WordPress</li>
    </ul>
    <h3 id="usage">Usage</h3>
    <p>Head over to the admin section of your site and click &quot;Security Headers&quot;, configure how you need it to be configured.  The configured headers will automatically be implemented.  Please be aware that some web hosts do not allow the header implementation utilized in this plugin (PHP), if you find this is the case for your site, head over to the "Implementation" page to see how you can accomplish this with other methods.</p>
    <p>Once configured you can try to have the content security policy automatically generated by getting into CLI on your sites server and running the following command: <code>wp csp generate</code></p>
    <p>That will attempt to parse your entire website and gather up all external domains tied to the resource it is attached to, then create a custom post to store the results in.</p>
    <p>You may need to run this more than once, and you will need to have some patience while it runs.</p>
    <p><strong>NOTE: </strong>If you are running this in a multi-site environment, you must pass the <code>--url</code> flag to the WP CLI command. Making the full command: <code>wp csp generate --url=YOUR_SITES_FULL_URL</code></p>
    <h3 id="settings">Settings</h3>
    <ul>
        <li>
            <strong>Content Security Policy</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin attempt to automatically generate a Content Security Policy. Selecting Yes, will open up a handful of text fields that you can add additional domains to. Please only include the FQDN,
                    and please organize them accordingly.
                    <ul>
                        <li>Use a space-delimited list if you need add more than one additional domain.</li>
                    </ul>
                </li>
                <li><p>This will parse the enqueued styles and scripts for the front-end of your site automatically, provided your theme has properly done this.</p></li>
                <li>
                    <p>This will add 2 additional headers to your site. Content-Security-Policy and X-Content-Security-Policy, configured with the detected and custom added domains to their respective content types.</p>
                </li>
                <li>See here for more information: <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/CSP" target="_blank">https://developer.mozilla.org/en-US/docs/Web/HTTP/CSP</a></li>
            </ul>
        </li>
        <li>
            <strong>Apply to Admin</strong>
            <ul>
                <li>Select yes or no to apply these same headers to the admin side of your site.</li>
            </ul>
        </li>
        <li>
            <strong>Wordpress Defaults</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin automatically include Wordpress specific external resources in the Content Security Policy
                    <ul>
                        <li>I have left this as an unconditional field. There may be a point in we browsers futures where it may have a use other than just the CSP</li>
                        <li>
                            <ul>
                                <li>img-src: *.googletagmanager.com *.w.org *.gravatar.com *.google.com *.google-analytics.com *.gstatic.com</li>
                                <li>script-src: *.g.doubleclick.net *.google-analytics.com *.google.com *.googletagmanager.com *.gstatic.com</li>
                                <li>style-src: *.googleapis.com *.gstatic.com</li>
                                <li>font-src: *.gstatic.com *.bootstrapcdn.com</li>
                                <li>frame-src: *.g.doubleclick.net *.google.com *.fls.doubleclick.net</li>
                                <li>connect-src: *.google-analytics.com *.wpengine.com yoast.com *.google.com *.g.doubleclick.net</li>
                            </ul>
                        </li>
                    </ul>
                </li>
            </ul>
        </li>
        <li>
            <strong>Strict Transport Security</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin include the Strict Transport Security header to your site.
                    <ul>
                        <li>
                            See here for more information:
                            <a target="_blank" href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Strict-Transport-Security">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Strict-Transport-Security</a>
                        </li>
                    </ul>
                </li>
                <li>
                    Configure the directives accordingly.
                    <ul>
                        <li>Cache Age: How long should browsers force HTTPS access for, in seconds.</li>
                        <li>Include Subdomains: Include all subdomains in this rule?</li>
                        <li>Preload: This is a Google specification. See here for more information: <a href="https://hstspreload.org/" target="_blank">https://hstspreload.org/</a></li>
                    </ul>
                </li>
            </ul>
        </li>
        <li>
            <strong>Frame Sources</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin add a header to configure frame sources.
                    <ul>
                        <li>
                            See here for more information: <a target="_blank" href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Frame-Options">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Frame-Options</a>
                        </li>
                    </ul>
                </li>
                <li>
                    Configure the directive to either decline all framing, or only allow SAMEORIGIN framing.
                </li>
            </ul>
        </li>
        <li>
            <strong>XSS Protection</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin add a header to try to prevent cross site scripting.
                    <ul>
                        <li>
                            See here for more information: <a target="_blank" href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-XSS-Protection">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-XSS-Protection</a>
                        </li>
                    </ul>
                </li>
            </ul>
        </li>
        <li>
            <strong>Prevent MimeType Sniffing</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin add a header to prevent mime-type sniffing.
                    <ul>
                        <li>
                            See here for more information:
                            <a target="_blank" href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Content-Type-Options">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Content-Type-Options</a>
                        </li>
                    </ul>
                </li>
            </ul>
        </li>
        <li>
            <strong>Origin Referrers</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin add a header to allow only origin referrers
                    <ul>
                        <li>
                            See here for more information: <a target="_blank" href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Referrer-Policy">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Referrer-Policy</a>
                        </li>
                    </ul>
                </li>
                <li>
                    Select Referrer Policy to send
                    <ul>
                        <li>No Referrer: The Referer header will be omitted entirely. No referrer information is sent along with requests.</li>
                        <li>Origin Only: Only send the origin of the document as the referrer.</li>
                        <li>Same Origin: A referrer will be sent for same-site origins, but cross-origin requests will send no referrer information.</li>
                        <li>Strict Origin on Cross Domain: Send the origin, path, and querystring when performing a same-origin request, only send the origin when the protocol security level stays the same while performing a cross-origin request</li>
                        <li>No Referrer on Downgrade: The origin, path, and querystring of the URL are sent as a referrer when the protocol security level stays the same</li>
                        <li>Origin on Cross Domain: Send the origin, path, and query string when performing a same-origin request, but only send the origin of the document for other cases.</li>
                        <li>Strict Origin: Only send the origin of the document as the referrer when the protocol security level stays the same</li>
                        <li>Full Referrer: Send the origin, path, and query string when performing any request, regardless of security.</li>
                    </ul>
                </li>
            </ul>
        </li>
        <li>
            <strong>Force Downloads</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin add a header to attempt to force downloading resources instead of directly openning them in the browser.
                    <ul>
                        <li>See here for more information: <a target="_blank" href="https://www.nwebsec.com/HttpHeaders/SecurityHeaders/XDownloadOptions">https://www.nwebsec.com/HttpHeaders/SecurityHeaders/XDownloadOptions</a></li>
                    </ul>
                </li>
            </ul>
        </li>
        <li>
            <strong>Cross Domain Origins</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin add a header to block cross domain origins. This generally is only
                    <ul>
                        <li>
                            See here for more information: <a target="_blank" href="https://webtechsurvey.com/response-header/x-permitted-cross-domain-policies">https://webtechsurvey.com/response-header/x-permitted-cross-domain-policies</a>
                        </li>
                    </ul>
                </li>
            </ul>
        </li>
        <li>
            <strong>Upgrade Insecure Requests</strong>
            <ul>
                <li>
                    Select Yes or No to have the plugin add a header to upgrade insecure requests.
                    <ul>
                        <li>
                            See here for more information:
                            <a target="_blank" href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Upgrade-Insecure-Requests">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Upgrade-Insecure-Requests</a>
                        </li>
                    </ul>
                </li>
            </ul>
        </li>
    </ul>
