<?php

// We don't want to allow direct access to this
defined( 'ABSPATH' ) OR die( 'No direct script access allowed' );

// utilize Carbon Fields
use Carbon_Fields\Container;
use Carbon_Fields\Field;

class KCP_CSPGEN_Settings {

    // do the work for this class
    public function kp_cspgen_settings( ) {

        // add in the menu
        $this -> kcp_cspgen_menu( );

    }

    // our menu items
    private function kcp_cspgen_menu( ) {

        // the parent admin page
        $_parent = Container::make( 'theme_options', __( 'Wordpress Security Header Generator Settings' ) )
            -> set_page_menu_title( 'Security Headers' )
            -> set_icon( 'dashicons-shield' )
            -> set_page_file( 'wpsh_settings' )
            -> set_page_menu_position( 2 )
            -> where( 'current_user_role', 'IN', array( 'administrator', 'super-admin' ) )
            -> add_fields( $this -> kcp_settings_fields( ) );

        // implementation
        Container::make( 'theme_options', __( 'Wordpress Security Header Generator Implementation' ) )
            -> set_page_menu_title( 'Implementation' )
            -> set_page_file( 'wpsh_implementation' )
            -> set_page_parent( $_parent )
            -> where( 'current_user_role', 'IN', array( 'administrator', 'super-admin' ) )
            -> add_fields( $this -> kcp_implementation_page( ) );

        // documentation
        Container::make( 'theme_options', __( 'Wordpress Security Header Generator Notes & Instructions' ) )
            -> set_page_menu_title( 'Documentation' )
            -> set_page_file( 'wpsh_documentation' )
            -> set_page_parent( $_parent )
            -> where( 'current_user_role', 'IN', array( 'administrator', 'super-admin' ) )
            -> add_fields( $this -> kcp_documentation_page( ) );

        // hook into the current screen and change the settings button
        add_action( 'current_screen', function( ) {

            // get the current screen
            global $current_screen;

            // switch for some extras
            switch( $current_screen -> base ) {
                case 'toplevel_page_wpsh_settings':

                    // change the save button content
                    add_action( 'admin_footer', function( ) {

                        // utilize jquery to replace the text in the button
                        echo '<script type="text/javascript">jQuery(".button-large").val("Update Your Settings");</script>';
                    } );
                    break;
                case 'security-headers_page_wpsh_implementation':
                case 'security-headers_page_wpsh_documentation':

                    // remove the metabox
                    add_action( 'admin_footer', function( ) {

                        // utilize jquery to replace the text in the button
                        echo '<script type="text/javascript">jQuery("#postbox-container-1").remove( );</script><style type="text/css">.columns-2 {margin-right:0px !important;}</style>';
                    } );

                    break;
            }

        } );

    }

    // populate the fields for the settings page
    private function kcp_settings_fields( ) : array {

        // setup the array to hold the fields
        $_ret = array(

            // apply to admin
            Field::make( 'checkbox', 'kp_cspgen_apply_to_admin', __( 'Do you want to apply these headers to your admin side as well?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'This will attempt to apply these headers to the admin side of your site in addition to the front-end.' ) ),
            
            // generate CSP
            Field::make( 'checkbox', 'kp_cspgen_generate_csp', __( 'Do you want to generate a Content Security Policy?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'Setting this will set the flag for generating a Content Security Policy.  See here for more information: <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/CSP" target="_blank">https://developer.mozilla.org/en-US/docs/Web/HTTP/CSP</a>' ) ),
           
            // include wordpress defaults: if generate CSP = true
            Field::make( 'checkbox', 'kp_cspgen_include_wordpress_defaults', __( 'Do you want to include Wordpress defaults?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'This will include resources for the following: <br />google.com, gstatic.com, google-analytics.com, googletagmanager.com, yoast.com, w.org, gravatar.com, doubleclick.net, bootstrapcdn.com, and wpengine.com' ) )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // basic auth username: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_auth_un', __( 'Basic Auth Username' ) )
                -> set_autoload( true )
                -> help_text( __( 'Enter your Basic Auth Username, if your site has this protection.' ) )
                -> set_classes( 'kpsh-half-field' )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),
            
            // basic auth password: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_auth_pw', __( 'Basic Auth Password' ) )
                -> help_text( __( 'Enter your Basic Auth Password, if your site has this protection.' ) )
                -> set_autoload( true )
                -> set_classes( 'kpsh-half-field' )
                -> set_attributes( array( 'type' => 'password' ) )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // style src: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_generate_csp_custom_styles', __( 'CSP Extra Style Domains' ) )
                -> help_text( __( 'Please enter a space delimited list of domains to add to your Content Security Policy.' ) )
                -> set_autoload( true )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // script src: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_generate_csp_custom_scripts', __( 'CSP Extra Script Domains' ) )
                -> help_text( __( 'Please enter a space delimited list of domains to add to your Content Security Policy.' ) )
                -> set_autoload( true )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // font src: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_generate_csp_custom_fonts', __( 'CSP Extra Font Domains' ) )
                -> help_text( __( 'Please enter a space delimited list of domains to add to your Content Security Policy.' ) )
                -> set_autoload( true )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // image src: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_generate_csp_custom_images', __( 'CSP Extra Image Domains' ) )
                -> help_text( __( 'Please enter a space delimited list of domains to add to your Content Security Policy.' ) )
                -> set_autoload( true )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // ajaxian src: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_generate_csp_custom_connect', __( 'CSP Extra Connect/Ajax Domains' ) )
                -> help_text( __( 'Please enter a space delimited list of domains to add to your Content Security Policy.' ) )
                -> set_autoload( true )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // frame src: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_generate_csp_custom_frames', __( 'CSP Extra Frame Domains' ) )
                -> help_text( __( 'Please enter a space delimited list of domains to add to your Content Security Policy.' ) )
                -> set_autoload( true )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // media src: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_generate_csp_custom_media', __( 'CSP Extra Media Domains' ) )
                -> help_text( __( 'Please enter a space delimited list of domains to add to your Content Security Policy.' ) )
                -> set_autoload( true )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // form src: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_generate_csp_custom_forms', __( 'CSP Extra Form Domains' ) )
                -> help_text( __( 'Please enter a space delimited list of domains to add to your Content Security Policy.' ) )
                -> set_autoload( true )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // default src: if generate CSP = true
            Field::make( 'text', 'kp_cspgen_generate_csp_custom_defaults', __( 'CSP Extra Default Domains' ) )
                -> help_text( __( 'Please enter a space delimited list of domains to add to your Content Security Policy.' ) )
                -> set_autoload( true )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_generate_csp',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // strict transport security header
            Field::make( 'checkbox', 'kp_cspgen_include_sts', __( 'Do you want to include Strict Transport Security?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'Setting this will add another header to force Strict Transport Security. See here for more information: <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Strict-Transport-Security" target="_blank">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Strict-Transport-Security</a>' ) ),
            
            // specify the max age: if STS is enabled
            Field::make( 'text', 'kp_cspgen_include_sts_max_age', __( 'Cache Age' ) )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_include_sts',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                )
                -> set_autoload( true )
                -> help_text( __( 'The time, in seconds, that the browser should remember that a site is only to be accessed using HTTPS.' ) )
                -> set_attributes( array( 'type' => 'number' ) )
                -> set_default_value( 31536000 ),
                
            // specify if STS should include subdomains: if STS is enabled
            Field::make( 'checkbox', 'kp_cspgen_include_sts_subdomains', __( 'Include Subdomains' ) )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_include_sts',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'If this optional parameter is specified, this rule applies to all of the site\'s subdomains as well.' ) )
                -> set_classes( 'kpsh-half-field' ),
            
            // specify if google's specs should preload
            Field::make( 'checkbox', 'kp_cspgen_include_sts_preload', __( 'Preload?' ) )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_include_sts',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> set_classes( 'kpsh-half-field' )
                -> help_text( __( 'If you enable preload, you should change the cache age to 2 Years. (63072000)' ) ),
        
            // origin frame sources
            Field::make( 'checkbox', 'kp_cspgen_include_ofs', __( 'Do you want to configure Frame Sources?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'Setting this will add another header to force SAMEORIGIN for frame sources. See here for more information: <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Frame-Options" target="_blank">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Frame-Options</a>' ) ),
            
            // deny or sameorigin frames
            Field::make( 'radio', 'kp_cspgen_include_ofs_type', __( 'Directives' ) )
                -> set_autoload( true )
                -> add_options(
                    array(
                        'DENY' => __( 'deny all framing' ),
                        'SAMEORIGIN' => __( 'deny all framing unless done from the origination domain' ),
                    )
                )
                -> set_classes( 'kpsh-half-field-li' )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_include_ofs',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),
            
            // block cross site scripting
            Field::make( 'checkbox', 'kp_cspgen_include_xss', __( 'Do you want to include XSS Protection?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'Setting this will add another header to for XSS protection. See here for more information: <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-XSS-Protection" target="_blank">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-XSS-Protection</a>' ) ),
            
            // prevent mimetype sniffing
            Field::make( 'checkbox', 'kp_cspgen_include_mimesniffing', __( 'Do you want to prevent mime-type sniffing?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'Setting this will add another header to for proper mime-types. See here for more information: <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Content-Type-Options" target="_blank">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/X-Content-Type-Options</a>' ) ),
            
            // include a referrer policy
            Field::make( 'checkbox', 'kp_cspgen_include_referrer_policy', __( 'Do you want to allow only origin referrers?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'Setting this will add another header to allow only origin referrers. See here for more information: <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Referrer-Policy" target="_blank">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Referrer-Policy</a>' ) ),
            
            // referrer directive
            Field::make( 'radio', 'kp_cspgen_include_referrer_policy_setting', __( 'Directives' ) )
                -> set_autoload( true )
                -> add_options(
                    array(
                        'no-referrer' => __( 'no referrer' ),
                        'no-referrer-when-downgrade' => __( 'no referrer on protocol downgrade' ),
                        'origin' => __( 'origin only' ),
                        'origin-when-cross-origin' => __( 'origin on cross-domain' ),
                        'same-origin' => __( 'same origin' ),
                        'strict-origin' => __( 'strict origin' ),
                        'strict-origin-when-cross-origin' => __( 'strict origin on cross domain' ),
                        'unsafe-url' => __( 'full referrer path' )
                    )
                )
                -> set_classes( 'kpsh-half-field-li' )
                -> set_conditional_logic( array( 
                    array(
                        'field' => 'kp_cspgen_include_referrer_policy',
                        'value' => true, // Optional, defaults to "". Should be an array if "IN" or "NOT IN" operators are used.
                        'compare' => '=', // Optional, defaults to "=". Available operators: =, <, >, <=, >=, IN, NOT IN
                    ), )
                ),

            // try to force downloads
            Field::make( 'checkbox', 'kp_cspgen_include_download_options', __( 'Do you want to force downloads?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'Setting this will add another header to downloading resources instead of directly openning them in the browser. See here for more information: <a href="https://www.nwebsec.com/HttpHeaders/SecurityHeaders/XDownloadOptions" target="_blank">https://www.nwebsec.com/HttpHeaders/SecurityHeaders/XDownloadOptions</a>' ) ),
            
            // block crossdomain pulls from PDF's and Flash
            Field::make( 'checkbox', 'kp_cspgen_include_crossdomain', __( 'Do you want to block cross domain origins?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'Setting this will add another header to block cross domain origins. See here for more information: <a href="https://webtechsurvey.com/response-header/x-permitted-cross-domain-policies" target="_blank">https://webtechsurvey.com/response-header/x-permitted-cross-domain-policies</a>' ) ),
            
            // upgrade insecure requests
            Field::make( 'checkbox', 'kp_cspgen_include_upgrade_insecure', __( 'Do you want to upgrade insecure requests?' ) )
                -> set_option_value( 'yes' )
                -> set_autoload( true )
                -> help_text( __( 'Setting this will add another header to upgrade insecure requests. See here for more information: <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Upgrade-Insecure-Requests" target="_blank">https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Upgrade-Insecure-Requests</a>' ) ),
                    
            );

        // return them
        return $_ret;

    }

    // implementation page "fields"
    private function kcp_implementation_page( ) : array {

        // setup the array to hold the fields
        $_ret = array( 
            Field::make( 'html', 'wpsh_implementation' )
                -> set_html( function( ) {

                    // include our implementation... because there is PHP processing, we need to utilize output bufferring
                    ob_start( );

                    // include the file
                    include_once( WPSH_PATH . '/work/manual.php' );

                    // throw the contents of the buffer into the out variable
                    $_out = ob_get_contents( );

                    // clean the output bufferring and end it
                    ob_end_clean( );

                    // return the rendered content
                    return $_out;
                } ),
        );

        // return them
        return $_ret;

    }

    // documentation and notes page
    private function kcp_documentation_page( ) : array {

        // setup the array to hold the fields
        $_ret = array( 
            Field::make( 'html', 'wpsh_documentation' )
                -> set_html( function( ) : string {

                    // include our documentation
                    return file_get_contents( WPSH_PATH . '/work/doc.php' );
                } ),
        );

        // return them
        return $_ret;

    }
 
}
